import os
from signal import SIGCHLD
from time import time

import gobject

import dbus
import dbus.service
import dbus.bus
import dbus.mainloop.glib

from rainbow import util, inject, permissions
from rainbow.util.linux.clone import clone, CLONE_NEWNS
from rainbow.gc import gc_spool

SPOOL = '/home/olpc/isolation/1'

def log(msg, *args):
    if len(args):
        print msg % args
    else:
        print msg

class Rainbow(dbus.service.Object):
    """The Rainbow security service."""
    SERVICE_NAME = 'org.laptop.security.Rainbow'
    INTERFACE_NAME = 'org.laptop.security.Rainbow'

    def __init__(self, bus_or_name):
        dbus.service.Object.__init__(self, bus_or_name, '/')
        self.preloader_hint = True
        self.preload_common_modules()

    def preload_common_modules(self):
        # Import-time logic is quite sensitive to environment variables.
        os.environ['XAUTHORITY'] = inject._XAUTHORITY
        os.environ['ICEAUTHORITY'] = inject._ICEAUTHORITY
        # Are we never going to want to launch python activities attached to other
        # displays? <MS>
        os.environ['DISPLAY'] = ':0'
        os.environ['GTK2_RC_FILES'] = '/usr/share/sugar/data/sugar-xo.gtkrc'

        imports = ('dbus.service',
                   'dbus.decorators',
                   'telepathy',
                   'sugar.activity.activity',
                   'sugar.activity.activityservice',
                   'sugar.graphics.style',
                   'sugar.graphics.window',
                   'rainbow.sugaractivity',
                  )

        start_time = time()

        for module in imports:
            try:
                __import__(module)
            except:
                print 'Exception when preloading module: %s' % module
                self.preloader_hint = False
                util.trace()
                break

        if self.preloader_hint:
            try:
                import gtk
                display = gtk.gdk.display_get_default()
                display.close() # This needs to happen after importing s.g.style
            except:
                print 'Exception when preloading module: gtk'
                self.preloader_hint = False
                util.trace()

        print 'Module preloading took %f seconds.' % (time() - start_time)


    @dbus.service.method(INTERFACE_NAME,
                         in_signature='s')
    def ChangeActivity(self, activity_id):
        """Sets CPU throttling on the foreground and background activities as needed."""
        pass

    @dbus.service.method(INTERFACE_NAME,
                         in_signature='sa{ss}asss',
                         async_callbacks=('success_cont', 'error_cont'),
                         utf8_strings=True)
    def CreateActivity(self, log_path, env, argv, bundle_path, bundle_id,
                       success_cont, error_cont):
        # Reap zombies
        try:
            while os.waitpid(-1, os.WNOHANG) != (0,0):
                pass
        except OSError:
            pass

        #
        # Try to pick up the capabilities/permissions from the
        #    bundle, if there isn't a permissions.info file, no
        #    big deal, the app just doesn't get extra permissions
        #
        pset = None
        permpath = os.path.join(bundle_path, "permissions.info")

        perm_worked = True
        try:
            f = open(permpath)
            pset = permissions.PermissionSet(f)
            f.close()
        except:
            perm_worked = False
            pset = None
            pass

        try:
            child_pid = clone(CLONE_NEWNS | SIGCHLD)
            if not child_pid:
                try:
                    # Close the connection to the system bus.
                    self.connection.close()

                    log_fd = os.open(log_path, os.O_WRONLY | os.O_SYNC)
                    os.dup2(log_fd, 1)
                    os.dup2(log_fd, 2)
                    ret = inject.run(log, SPOOL, env, argv, env['SUGAR_BUNDLE_PATH'], pset, (1, 2),
                            env.get('RAINBOW_STRACE_LOG'), 500, 500, bundle_path, bundle_id,
                            env.get('RAINBOW_CONSTANT_UID'), self.preloader_hint)
                except Exception, e:
                    util.trace()
                    error_cont(e)
                finally:
                    os._exit(255)

            success_cont()

        except Exception, e:
            util.trace()
            error_cont(e)

def run(opts, args):
    """Start the Rainbow DBus service."""
    log("GC'ing spool %s", SPOOL)
    try: gc_spool(log, SPOOL)
    except: util.trace()

    loop = dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    # Seems like we need to set this before we create a connection, or the old
    # value would persist after the fork.
    os.environ['DBUS_SESSION_BUS_ADDRESS'] = 'unix:path=/tmp/olpc-session-bus'

    bus = dbus.SystemBus(private=True, mainloop=loop)
    name = dbus.service.BusName(Rainbow.SERVICE_NAME, bus)
    Rainbow(name)

    print 'Service running mainloop.'
    mainloop = gobject.MainLoop()
    mainloop.run()
