# Code from http://osdir.com/ml/python.ctypes/2006-12/msg00032.html

import os
import errno
import sys
from ctypes import *
from ctypes.util import find_library

libc = CDLL(find_library('c'))
libc.__errno_location.restype = POINTER(c_int)

def get_errno():
    return libc.__errno_location().contents.value

class CError(Exception):
    """Base class for VServer syscall errors."""

    def __init__(self, errno_=None):
        if errno_ is None:
            errno_ = get_errno()
        self.errno = errno_
        self.errsym = errno.errorcode[errno_]
        self.errstring = os.strerror(errno_)

    def __repr__(self):
        return '<%s errno=%s>'%(self.__class__.__name__, self.errno)

    def __str__(self):
        return '%s: %s'%(self.errsym, self.errstring)

    @staticmethod
    def errcheck(result, func=None, args=None):
        """Use with ctypes._CFuncPtr, e.g.
        func.errcheck = CError.errcheck
        """
        if func is not None and hasattr(func, 'errval'):
            if result == func.errval:
                raise CError(get_errno())
        else:
            if result != 0:
                raise CError(get_errno())

        return result
    
    @staticmethod
    def check(errval, result):
        """Use with ctypes._CFuncPtr, e.g.
        func.errcheck = CError.errcheck
        """
        if result == errval:
            raise CError(get_errno())

        return result
