from ctypes import *
from ctypes.util import find_library

from .c_errno import CError, errno_errcheck

class MountError(CError):
    pass

libc = CDLL(find_library('c'))

MS_RDONLY = 1           # Mount read-only. 
MS_NOSUID = 2           # Ignore suid and sgid bits. 
MS_NODEV = 4            # Disallow access to device special files.  
MS_NOEXEC = 8           # Disallow program execution.  
MS_SYNCHRONOUS = 16     # Writes are synced at once.  
MS_REMOUNT = 32         # Alter flags of a mounted FS.  
MS_MANDLOCK = 64        # Allow mandatory locks on an FS.  
S_WRITE = 128           # Write on file/directory/symlink.  
S_APPEND = 256          # Append-only file.  
S_IMMUTABLE = 512       # Immutable file.  
MS_NOATIME = 1024       # Do not update access times.  
MS_NODIRATIME = 2048    # Do not update directory access times.  
MS_BIND = 4096          # Bind directory at different place.  
MS_REC = 16384          # Recursive mount (use with MS_BIN).

MS_MGC_VAL = 0xc0ed0000 # Magic flag number to indicate "new" flags
MS_MGC_MSK = 0xffff0000 # Magic flag number mask

#extern int mount (__const char *__special_file, __const char *__dir,
#          __const char *__fstype, unsigned long int __rwflag,
#          __const void *__data) __THROW;

mount = libc.mount
mount.__doc__ = "Libc's mount call; see mount (2)."
mount.argtypes = [c_char_p, c_char_p, c_char_p, c_uint32, c_char_p]
mount.restype = c_int
mount.errval = -1
mount.errcheck = errno_errcheck(mount.errval, CError)

#extern int umount (__const char *__special_file) __THROW;
umount = libc.umount
umount.__doc__ = "Libc's umount call; see umount (2)."
umount.argtypes = [c_char_p]
umount.restype = c_int
umount.errval = -1
umount.errcheck = errno_errcheck(umount.errval, CError)


def bindmount(src, tgt, read_only, recursive):
    """Calls mount with appropriate arguments.
    """
    flags = MS_BIND
    if read_only:
        flags |= MS_RDONLY
    if recursive:
        flags |= MS_REC
    mount(src, tgt, '', flags, '')

